@@ -1,26 +1,27 @@
-/*
-
-Copyright (C) 1996-2018 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
-
-// Author: Steven G. Johnson <stevenj@alum.mit.edu>
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2023 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -31,8 +32,9 @@ along with Octave; see the file COPYING.  If not, see
 #include <cctype>
 
 #include <iomanip>
-#include <iostream>
+#include <istream>
 #include <limits>
+#include <ostream>
 #include <string>
 #include <vector>
 
@@ -52,6 +54,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "defun.h"
 #include "error.h"
 #include "errwarn.h"
+#include "interpreter.h"
 #include "interpreter-private.h"
 #include "load-save.h"
 #include "oct-hdf5.h"
@@ -59,7 +62,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-map.h"
 #include "ov-cell.h"
 #include "pager.h"
-#include "pt-exp.h"
 #include "sysdep.h"
 #include "unwind-prot.h"
 #include "utils.h"
@@ -88,19 +90,7 @@ hdf5_fstreambase::hdf5_fstreambase (const char *name, int mode, int /* prot */)
   : file_id (-1), current_item (-1)
 {
 #if defined (HAVE_HDF5)
-
-  if (mode & std::ios::in)
-    file_id = H5Fopen (name, H5F_ACC_RDONLY, octave_H5P_DEFAULT);
-  else if (mode & std::ios::out)
-    {
-      if (mode & std::ios::app && H5Fis_hdf5 (name) > 0)
-        file_id = H5Fopen (name, H5F_ACC_RDWR, octave_H5P_DEFAULT);
-      else
-        file_id = H5Fcreate (name, H5F_ACC_TRUNC, octave_H5P_DEFAULT,
-                             octave_H5P_DEFAULT);
-    }
-  if (file_id < 0)
-    std::ios::setstate (std::ios::badbit);
+  open_create (name, mode);
 
   current_item = 0;
 
@@ -136,20 +126,109 @@ hdf5_fstreambase::open (const char *name, int mode, int)
 
   clear ();
 
+  open_create (name, mode);
+
+  current_item = 0;
+
+#else
+  // This shouldn't happen because construction of hdf5_fstreambase
+  // objects is supposed to be impossible if HDF5 is not available.
+
+  panic_impossible ();
+#endif
+}
+
+void
+hdf5_fstreambase::open_create (const char *name, int mode)
+{
+#if defined (HAVE_HDF5)
+  // Open the HDF5 file NAME.  If it does not exist, create the file.
+
+#  if defined (HAVE_HDF5_UTF8)
+  const char *fname = name;
+#  else
+  std::string fname_str (name);
+  std::string ascii_fname_str = octave::sys::get_ASCII_filename (fname_str);
+  const char *fname = ascii_fname_str.c_str ();
+#  endif
+
   if (mode & std::ios::in)
-    file_id = H5Fopen (name, H5F_ACC_RDONLY, octave_H5P_DEFAULT);
+    file_id = H5Fopen (fname, H5F_ACC_RDONLY, octave_H5P_DEFAULT);
   else if (mode & std::ios::out)
     {
-      if (mode & std::ios::app && H5Fis_hdf5 (name) > 0)
-        file_id = H5Fopen (name, H5F_ACC_RDWR, octave_H5P_DEFAULT);
+      if (mode & std::ios::app && H5Fis_hdf5 (fname) > 0)
+        file_id = H5Fopen (fname, H5F_ACC_RDWR, octave_H5P_DEFAULT);
       else
-        file_id = H5Fcreate (name, H5F_ACC_TRUNC, octave_H5P_DEFAULT,
+#  if defined (HAVE_HDF5_UTF8)
+        file_id = H5Fcreate (fname, H5F_ACC_TRUNC, octave_H5P_DEFAULT,
                              octave_H5P_DEFAULT);
+#  else
+        {
+          // Check whether file already exists
+          std::string abs_ascii_fname
+            = octave::sys::canonicalize_file_name (ascii_fname_str);
+          if (! abs_ascii_fname.empty ())
+            {
+              // Use the existing file
+              file_id = H5Fcreate (fname, H5F_ACC_TRUNC,
+                                   octave_H5P_DEFAULT, octave_H5P_DEFAULT);
+              if (file_id < 0)
+                std::ios::setstate (std::ios::badbit);
+
+              return;
+            }
+
+          // Check whether filename contains non-ASCII (UTF-8) characters.
+          std::string::const_iterator first_non_ASCII
+            = std::find_if (fname_str.begin (), fname_str.end (),
+          [](char c) { return (c < 0 || c >= 128); });
+          if (first_non_ASCII == fname_str.end ())
+            {
+              // No non-ASCII characters
+              file_id = H5Fcreate (name, H5F_ACC_TRUNC, octave_H5P_DEFAULT,
+                                   octave_H5P_DEFAULT);
+              if (file_id < 0)
+                std::ios::setstate (std::ios::badbit);
+
+              return;
+            }
+
+          // Create file in temp folder
+          std::string tmp_name = octave::sys::tempnam ("", "oct-");
+          octave_hdf5_id hdf5_fid = H5Fcreate (tmp_name.c_str (), H5F_ACC_TRUNC,
+                                               octave_H5P_DEFAULT,
+                                               octave_H5P_DEFAULT);
+          if (hdf5_fid < 0)
+            {
+              file_id = -1;
+              std::ios::setstate (std::ios::badbit);
+              return;
+            }
+
+          // Close file
+          H5Fclose (hdf5_fid);
+
+          // Move temporary file to final destination
+          std::string msg;
+          int res = octave::sys::rename (tmp_name, name, msg);
+          if (res < 0)
+            {
+              std::ios::setstate (std::ios::badbit);
+              file_id = -1;
+              return;
+            }
+
+          // Open file at final location
+          ascii_fname_str = octave::sys::get_ASCII_filename (fname_str);
+          ascii_fname = ascii_fname_str.c_str ();
+          file_id = H5Fopen (ascii_fname, H5F_ACC_RDWR, octave_H5P_DEFAULT);
+        }
+#  endif
     }
   if (file_id < 0)
     std::ios::setstate (std::ios::badbit);
 
-  current_item = 0;
+  return;
 
 #else
   // This shouldn't happen because construction of hdf5_fstreambase
@@ -164,14 +243,14 @@ make_valid_identifier (const std::string& nm)
 {
   std::string retval;
 
-  size_t nm_len = nm.length ();
+  std::size_t nm_len = nm.length ();
 
   if (nm_len > 0)
     {
       if (! isalpha (nm[0]))
         retval += '_';
 
-      for (size_t i = 0; i < nm_len; i++)
+      for (std::size_t i = 0; i < nm_len; i++)
         {
           char c = nm[i];
           retval += (isalnum (c) || c == '_') ? c : '_';
@@ -228,17 +307,17 @@ hdf5_check_attr (octave_hdf5_id loc_id, const char *attr_name)
   // HDF5 doesn't print out all sorts of error messages if we
   // call H5Aopen for a non-existing attribute
 
-  H5E_auto_t err_func;
-  void *err_func_data;
+  H5E_auto_t err_fcn;
+  void *err_fcn_data;
 
   // turn off error reporting temporarily, but save the error
   // reporting function:
 
 #if defined (HAVE_HDF5_18)
-  H5Eget_auto (octave_H5E_DEFAULT, &err_func, &err_func_data);
+  H5Eget_auto (octave_H5E_DEFAULT, &err_fcn, &err_fcn_data);
   H5Eset_auto (octave_H5E_DEFAULT, nullptr, nullptr);
 #else
-  H5Eget_auto (&err_func, &err_func_data);
+  H5Eget_auto (&err_fcn, &err_fcn_data);
   H5Eset_auto (nullptr, nullptr);
 #endif
 
@@ -253,9 +332,9 @@ hdf5_check_attr (octave_hdf5_id loc_id, const char *attr_name)
 
   // restore error reporting:
 #if defined (HAVE_HDF5_18)
-  H5Eset_auto (octave_H5E_DEFAULT, err_func, err_func_data);
+  H5Eset_auto (octave_H5E_DEFAULT, err_fcn, err_fcn_data);
 #else
-  H5Eset_auto (err_func, err_func_data);
+  H5Eset_auto (err_fcn, err_fcn_data);
 #endif
   return retval;
 
@@ -276,17 +355,17 @@ hdf5_get_scalar_attr (octave_hdf5_id loc_id, octave_hdf5_id type_id,
   // HDF5 doesn't print out all sorts of error messages if we
   // call H5Aopen for a non-existing attribute
 
-  H5E_auto_t err_func;
-  void *err_func_data;
+  H5E_auto_t err_fcn;
+  void *err_fcn_data;
 
   // turn off error reporting temporarily, but save the error
   // reporting function:
 
 #if defined (HAVE_HDF5_18)
-  H5Eget_auto (octave_H5E_DEFAULT, &err_func, &err_func_data);
+  H5Eget_auto (octave_H5E_DEFAULT, &err_fcn, &err_fcn_data);
   H5Eset_auto (octave_H5E_DEFAULT, nullptr, nullptr);
 #else
-  H5Eget_auto (&err_func, &err_func_data);
+  H5Eget_auto (&err_fcn, &err_fcn_data);
   H5Eset_auto (nullptr, nullptr);
 #endif
 
@@ -305,9 +384,9 @@ hdf5_get_scalar_attr (octave_hdf5_id loc_id, octave_hdf5_id type_id,
 
   // restore error reporting:
 #if defined (HAVE_HDF5_18)
-  H5Eset_auto (octave_H5E_DEFAULT, err_func, err_func_data);
+  H5Eset_auto (octave_H5E_DEFAULT, err_fcn, err_fcn_data);
 #else
-  H5Eset_auto (err_func, err_func_data);
+  H5Eset_auto (err_fcn, err_fcn_data);
 #endif
   return retval;
 
@@ -341,6 +420,229 @@ hdf5_make_complex_type (octave_hdf5_id num_type)
 
 #if defined (HAVE_HDF5)
 
+// The following subroutine creates an HDF5 representation of the way
+// we will store Octave range types (triplets of floating-point numbers).
+// NUM_TYPE is the HDF5 numeric type to use for storage
+// (e.g., H5T_NATIVE_DOUBLE to save as 'double').
+// Note that any necessary conversions are handled automatically by HDF5.
+
+static hid_t
+hdf5_make_range_type (hid_t num_type)
+{
+  hid_t type_id = H5Tcreate (H5T_COMPOUND, sizeof (double) * 3);
+
+  H5Tinsert (type_id, "base", 0 * sizeof (double), num_type);
+  H5Tinsert (type_id, "limit", 1 * sizeof (double), num_type);
+  H5Tinsert (type_id, "increment", 2 * sizeof (double), num_type);
+
+  return type_id;
+}
+
+static herr_t
+load_inline_fcn (hid_t loc_id, const char *name, octave_value& retval)
+{
+#if defined (HAVE_HDF5)
+
+  hid_t group_hid, data_hid, space_hid, type_hid, type_class_hid, st_id;
+  hsize_t rank;
+  int slen;
+
+#if defined (HAVE_HDF5_18)
+  group_hid = H5Gopen (loc_id, name, octave_H5P_DEFAULT);
+#else
+  group_hid = H5Gopen (loc_id, name);
+#endif
+  if (group_hid < 0) return -1;
+
+#if defined (HAVE_HDF5_18)
+  data_hid = H5Dopen (group_hid, "args", octave_H5P_DEFAULT);
+#else
+  data_hid = H5Dopen (group_hid, "args");
+#endif
+  space_hid = H5Dget_space (data_hid);
+  rank = H5Sget_simple_extent_ndims (space_hid);
+
+  if (rank != 2)
+    {
+      H5Dclose (data_hid);
+      H5Sclose (space_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  OCTAVE_LOCAL_BUFFER (hsize_t, hdims, rank);
+  OCTAVE_LOCAL_BUFFER (hsize_t, maxdims, rank);
+
+  H5Sget_simple_extent_dims (space_hid, hdims, maxdims);
+
+  octave_value_list args (hdims[1]+1);
+
+  OCTAVE_LOCAL_BUFFER (char, s1, hdims[0] * hdims[1]);
+
+  if (H5Dread (data_hid, H5T_NATIVE_UCHAR, octave_H5S_ALL, octave_H5S_ALL,
+               octave_H5P_DEFAULT, s1) < 0)
+    {
+      H5Dclose (data_hid);
+      H5Sclose (space_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  H5Dclose (data_hid);
+  H5Sclose (space_hid);
+
+  for (std::size_t i = 0; i < hdims[1]; i++)
+    args(i+1) = std::string (s1 + i*hdims[0]);
+
+#if defined (HAVE_HDF5_18)
+  data_hid = H5Dopen (group_hid, "nm", octave_H5P_DEFAULT);
+#else
+  data_hid = H5Dopen (group_hid, "nm");
+#endif
+
+  if (data_hid < 0)
+    {
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  type_hid = H5Dget_type (data_hid);
+  type_class_hid = H5Tget_class (type_hid);
+
+  if (type_class_hid != H5T_STRING)
+    {
+      H5Tclose (type_hid);
+      H5Dclose (data_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  space_hid = H5Dget_space (data_hid);
+  rank = H5Sget_simple_extent_ndims (space_hid);
+
+  if (rank != 0)
+    {
+      H5Sclose (space_hid);
+      H5Tclose (type_hid);
+      H5Dclose (data_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  slen = H5Tget_size (type_hid);
+  if (slen < 0)
+    {
+      H5Sclose (space_hid);
+      H5Tclose (type_hid);
+      H5Dclose (data_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  OCTAVE_LOCAL_BUFFER (char, nm_tmp, slen);
+
+  // create datatype for (null-terminated) string to read into:
+  st_id = H5Tcopy (H5T_C_S1);
+  H5Tset_size (st_id, slen);
+
+  if (H5Dread (data_hid, st_id, octave_H5S_ALL, octave_H5S_ALL,
+               octave_H5P_DEFAULT, nm_tmp) < 0)
+    {
+      H5Sclose (space_hid);
+      H5Tclose (type_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+  H5Tclose (st_id);
+  H5Dclose (data_hid);
+  // NAME is obsolete and unused.
+  // std::string name (nm_tmp);
+
+#if defined (HAVE_HDF5_18)
+  data_hid = H5Dopen (group_hid, "iftext", octave_H5P_DEFAULT);
+#else
+  data_hid = H5Dopen (group_hid, "iftext");
+#endif
+
+  if (data_hid < 0)
+    {
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  type_hid = H5Dget_type (data_hid);
+  type_class_hid = H5Tget_class (type_hid);
+
+  if (type_class_hid != H5T_STRING)
+    {
+      H5Tclose (type_hid);
+      H5Dclose (data_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  space_hid = H5Dget_space (data_hid);
+  rank = H5Sget_simple_extent_ndims (space_hid);
+
+  if (rank != 0)
+    {
+      H5Sclose (space_hid);
+      H5Tclose (type_hid);
+      H5Dclose (data_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  slen = H5Tget_size (type_hid);
+  if (slen < 0)
+    {
+      H5Sclose (space_hid);
+      H5Tclose (type_hid);
+      H5Dclose (data_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+
+  OCTAVE_LOCAL_BUFFER (char, iftext_tmp, slen);
+
+  // create datatype for (null-terminated) string to read into:
+  st_id = H5Tcopy (H5T_C_S1);
+  H5Tset_size (st_id, slen);
+
+  if (H5Dread (data_hid, st_id, octave_H5S_ALL, octave_H5S_ALL,
+               octave_H5P_DEFAULT, iftext_tmp) < 0)
+    {
+      H5Sclose (space_hid);
+      H5Tclose (type_hid);
+      H5Gclose (group_hid);
+      return -1;
+    }
+  H5Tclose (st_id);
+  H5Dclose (data_hid);
+
+  args(0) = std::string (iftext_tmp);
+
+  octave::interpreter& interp = octave::__get_interpreter__ ();
+
+  octave_value_list tmp = interp.feval ("inline", args, 1);
+
+  if (tmp.length () > 0)
+    {
+      retval = tmp(0);
+      return 1;
+    }
+
+#else
+  octave_unused_parameter (loc_id);
+  octave_unused_parameter (name);
+  octave_unused_parameter (retval);
+
+  warn_load ("hdf5");
+#endif
+
+  return -1;
+}
+
 // This function is designed to be passed to H5Giterate, which calls it
 // on each data item in an HDF5 file.  For the item whose name is NAME in
 // the group GROUP_ID, this function sets dv->tc to an Octave representation
@@ -350,6 +652,8 @@ hdf5_make_complex_type (octave_hdf5_id num_type)
 // It returns 1 on success (in which case H5Giterate stops and returns),
 // -1 on error, and 0 to tell H5Giterate to continue on to the next item
 // (e.g., if NAME was a data type we don't recognize).
+//
+// This function must not throw an exception.
 
 static herr_t
 hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
@@ -363,12 +667,11 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
 
   H5G_stat_t info;
   herr_t retval = 0;
-  bool ident_valid = valid_identifier (name);
+  bool ident_valid = octave::valid_identifier (name);
 
   std::string vname = name;
 
-  octave::type_info& type_info
-    = octave::__get_type_info__ ("hdf5_read_next_data_internal");
+  octave::type_info& type_info = octave::__get_type_info__ ();
 
   // Allow identifiers as all digits so we can load lists saved by
   // earlier versions of Octave.
@@ -379,7 +682,7 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
       vname = make_valid_identifier (vname);
 
       // check again (in case vname was null, empty, or some such thing):
-      ident_valid = valid_identifier (vname);
+      ident_valid = octave::valid_identifier (vname);
     }
 
   H5Gget_objinfo (group_id, name, 1, &info);
@@ -435,16 +738,30 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
           hid_t st_id = H5Tcopy (H5T_C_S1);
           H5Tset_size (st_id, slen);
 
-          if (H5Dread (data_id, st_id, octave_H5S_ALL, octave_H5S_ALL, octave_H5P_DEFAULT,
-                       typ) < 0)
+          if (H5Dread (data_id, st_id, octave_H5S_ALL, octave_H5S_ALL,
+                       octave_H5P_DEFAULT, typ) < 0)
             goto done;
 
           H5Tclose (st_id);
           H5Dclose (data_id);
 
-          d->tc = type_info.lookup_type (typ);
+          if (std::string (typ, slen-1) == "inline function")
+            {
+              retval = load_inline_fcn (subgroup_id, name, d->tc);
+            }
+          else
+            {
+              d->tc = type_info.lookup_type (std::string (typ, slen-1));
 
-          retval = (d->tc.load_hdf5 (subgroup_id, "value") ? 1 : -1);
+              try
+                {
+                  retval = (d->tc.load_hdf5 (subgroup_id, "value") ? 1 : -1);
+                }
+              catch (const octave::execution_exception& ee)
+                {
+                  retval = -1;
+                }
+            }
 
           // check for OCTAVE_GLOBAL attribute:
           d->global = hdf5_check_attr (subgroup_id, "OCTAVE_GLOBAL");
@@ -453,6 +770,10 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
         }
       else
         {
+          // It seems that this block only applies to an old list type
+          // and that we shouldn't need to handle the old inline
+          // function type here.
+
           // an HDF5 group is treated as an octave structure by
           // default (since that preserves name information), and an
           // octave list otherwise.
@@ -467,12 +788,23 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
 
           H5Gclose (subgroup_id);
 
-          retval = (d->tc.load_hdf5 (group_id, name) ? 1 : -1);
+          try
+            {
+              retval = (d->tc.load_hdf5 (group_id, name) ? 1 : -1);
+            }
+          catch (const octave::execution_exception& ee)
+            {
+              retval = -1;
+            }
         }
 
     }
   else if (info.type == H5G_DATASET && ident_valid)
     {
+      // It seems that this block only applies to an old version of
+      // Octave HDF5 files and that it is probably not important to
+      // handle the old inline function type here.
+
       // For backwards compatibility.
 #if defined (HAVE_HDF5_18)
       data_id = H5Dopen (group_id, name, octave_H5P_DEFAULT);
@@ -613,6 +945,7 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
       else if (type_class_id == H5T_COMPOUND)
         {
           hid_t complex_type = hdf5_make_complex_type (H5T_NATIVE_DOUBLE);
+          hid_t range_type = hdf5_make_range_type (H5T_NATIVE_DOUBLE);
 
           if (hdf5_types_compatible (type_id, complex_type))
             {
@@ -627,27 +960,42 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
 
               H5Sclose (space_id);
             }
-          else
-            // Assume that if its not complex its a range.
-            // If its not, it'll be rejected later in the range code.
-            d->tc = type_info.lookup_type ("range");
+          else if (hdf5_types_compatible (type_id, range_type))
+            {
+              // If it's not a complex, check if it's a range
+              d->tc = octave_value_typeinfo::lookup_type ("range");
+            }
+          else // Otherwise, just ignore it with a warning.
+            {
+              warning ("load: can't read '%s' (unknown datatype)", name);
+              retval = 0;  // unknown datatype; skip
+              return retval;
+            }
 
+          H5Tclose (range_type);
           H5Tclose (complex_type);
         }
       else
         {
           warning ("load: can't read '%s' (unknown datatype)", name);
-          retval = 0; // unknown datatype; skip
+          retval = 0;  // unknown datatype; skip
           return retval;
         }
 
       // check for OCTAVE_GLOBAL attribute:
       d->global = hdf5_check_attr (data_id, "OCTAVE_GLOBAL");
 
+      try
+        {
+          retval = (d->tc.load_hdf5 (group_id, name) ? 1 : -1);
+        }
+      catch (const octave::execution_exception& ee)
+        {
+          retval = -1;
+        }
+
       H5Tclose (type_id);
       H5Dclose (data_id);
-
-      retval = (d->tc.load_hdf5 (group_id, name) ? 1 : -1);
     }
 
   if (! ident_valid)
@@ -660,7 +1008,11 @@ hdf5_read_next_data_internal (hid_t group_id, const char *name, void *dv)
 
 done:
   if (retval < 0)
-    error ("load: error while reading hdf5 item %s", name);
+    {
+      // Must be warning.  A call to error aborts and leaves H5Giterate in
+      // a mangled state that causes segfault on exit (bug #56149).
+      warning ("load: error while reading hdf5 item '%s'", name);
+    }
 
   if (retval > 0)
     {
@@ -720,7 +1072,7 @@ hdf5_h5g_iterate (octave_hdf5_id loc_id, const char *name, int *idx,
 }
 
 // Read the next Octave variable from the stream IS, which must really be an
-// hdf5_ifstream.  Return the variable value in tc, its doc string in doc, and
+// hdf5_ifstream.  Return the variable value in tc, its docstring in doc, and
 // whether it is global in global.  The return value is the name of the
 // variable, or NULL if none were found or there was an error.
 std::string
@@ -730,7 +1082,7 @@ read_hdf5_data (std::istream& is, const std::string& /* filename */,
 {
 #if defined (HAVE_HDF5)
 
-  check_hdf5_types ();
+  octave::check_hdf5_types ();
 
   std::string retval;
 
@@ -757,7 +1109,7 @@ read_hdf5_data (std::istream& is, const std::string& /* filename */,
     {
       std::vector<char> var_name;
       bool found = false;
-      size_t len = 0;
+      std::size_t len = 0;
 
       len = H5Gget_objname_by_idx (hs.file_id, hs.current_item, nullptr, 0);
       var_name.resize (len+1);
@@ -791,9 +1143,9 @@ read_hdf5_data (std::istream& is, const std::string& /* filename */,
     }
   else
     {
-      // an error occurred (H5Giterate_retval < 0) or there are no
-      // more datasets print an error message if retval < 0?
-      // hdf5_read_next_data already printed one, probably.
+      // An error occurred (H5Giterate_retval < 0),
+      // or there are no more datasets (H5Giterate_retval == 0).
+      // hdf5_read_next_data_internal has already printed a warning msg.
     }
 
   if (! d.name.empty ())
@@ -918,7 +1270,8 @@ save_hdf5_empty (octave_hdf5_id loc_id, const char *name, const dim_vector& d)
   if (space_hid < 0) return space_hid;
 #if defined (HAVE_HDF5_18)
   data_hid = H5Dcreate (loc_id, name, H5T_NATIVE_IDX, space_hid,
-                        octave_H5P_DEFAULT, octave_H5P_DEFAULT, octave_H5P_DEFAULT);
+                        octave_H5P_DEFAULT, octave_H5P_DEFAULT,
+                        octave_H5P_DEFAULT);
 #else
   data_hid = H5Dcreate (loc_id, name, H5T_NATIVE_IDX, space_hid,
                         octave_H5P_DEFAULT);
@@ -935,7 +1288,7 @@ save_hdf5_empty (octave_hdf5_id loc_id, const char *name, const dim_vector& d)
   H5Dclose (data_hid);
   H5Sclose (space_hid);
 
-  if (retval >= 0)
+  if (retval)
     retval = hdf5_add_attr (loc_id, "OCTAVE_EMPTY_MATRIX");
 
   return (retval == 0 ? 1 : retval);
@@ -1085,7 +1438,7 @@ add_hdf5_data (octave_hdf5_id loc_id, const octave_value& tc,
     goto error_cleanup;
 
   dims[0] = 0;
-  space_id = H5Screate_simple (0 , dims, nullptr);
+  space_id = H5Screate_simple (0, dims, nullptr);
   if (space_id < 0)
     goto error_cleanup;
 #if defined (HAVE_HDF5_18)
@@ -1151,7 +1504,7 @@ save_hdf5_data (std::ostream& os, const octave_value& tc,
 {
 #if defined (HAVE_HDF5)
 
-  check_hdf5_types ();
+  octave::check_hdf5_types ();
 
   hdf5_ofstream& hs = dynamic_cast<hdf5_ofstream&> (os);
 
