@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2016-2021 The Octave Project Developers
+// Copyright (C) 2016-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -30,7 +30,7 @@
 extern "C" {
 #endif
 
-extern int octave_mkostemps_wrapper (char *tmpl, int suffixlen);
+extern OCTAVE_API int octave_mkostemps_wrapper (char *tmpl, int suffixlen);
 
 #if defined __cplusplus
 }
