@@ -1,4 +1,9 @@
-## Copyright (C) 2017-2019 John W. Eaton
+########################################################################
+##
+## Copyright (C) 2017-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,13 +20,11 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 function disp (obj)
 
-  if (nargin != 1)
-    print_usage ();
-  endif
-
   printf ("  FTP object\n");
   printf ("    host: %s\n", obj.host);
   printf ("    user: %s\n", obj.username);
