@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2014-2020 The Octave Project Developers
+## Copyright (C) 2014-2023 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -26,6 +26,7 @@
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{map} =} cubehelix ()
 ## @deftypefnx {} {@var{map} =} cubehelix (@var{n})
+## @deftypefnx {} {@var{map} =} cubehelix (@var{n}, @var{start}, @var{rots}, @var{hue}, @var{gamma})
 ## Create cubehelix colormap.
 ##
 ## This colormap varies from black to white going though blue, green, and red
@@ -50,9 +51,7 @@
 
 function map = cubehelix (n, start = 0.5, rots = -1.5, hue = 1, gamma = 1)
 
-  if (nargin > 5)
-    print_usage ();
-  elseif (nargin > 0)
+  if (nargin > 0)
     if (! isscalar (n))
       error ("cubehelix: N must be a scalar");
     endif
@@ -88,7 +87,7 @@ function map = cubehelix (n, start = 0.5, rots = -1.5, hue = 1, gamma = 1)
 endfunction
 
 
-## A better demo of this colormap would be a 3D plot in NTSC instead of
+## A better demo of this colormap would be a 3-D plot in NTSC instead of
 ## RGB values.  That would really show what this colormap is about.
 %!demo
 %! ## Show the 'cubehelix' colormap profile and as an image
