@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2008-2021 The Octave Project Developers
+## Copyright (C) 2008-2023 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -220,7 +220,7 @@ function h = colorbar (varargin)
     set (hax, "units", orig_props.units,
               "position", orig_props.position,
               "outerposition", orig_props.outerposition,
-              "activepositionproperty", orig_props.activepositionproperty);
+              "positionconstraint", orig_props.positionconstraint);
     set (hax, "units", units);
   endif
 
@@ -252,7 +252,7 @@ function h = colorbar (varargin)
     ## FIXME: Matlab does not require the "position" property to be active.
     ##        Is there a way to determine the plotbox position for the
     ##        gnuplot graphics toolkit when the outerposition is active?
-    set (hax, "activepositionproperty", "position");
+    set (hax, "positionconstraint", "innerposition");
     props = get (hax);
     props.__axes_handle__ = hax;
     position = props.position;
@@ -277,7 +277,7 @@ function h = colorbar (varargin)
     ## Create colorbar axes if necessary
     if (new_colorbar)
       hcb = axes ("parent", hpar, "tag", "colorbar",
-                  "activepositionproperty", "position",
+                  "positionconstraint", "innerposition",
                   "units", get (hax, "units"), "position", cbpos,
                   "colormap", cmap,
                   "box", "on", "xdir", "normal", "ydir", "normal");
@@ -294,7 +294,7 @@ function h = colorbar (varargin)
       addproperty ("tickdirection", hcb, "AxesTickdir", "in");
       ## FIXME: Matlab uses just a scalar for ticklength, but axes already
       ##        has a 2-element ticklength property which cannot be overridden.
-      ##addproperty ("ticklength", hcb, "double", 0.01);
+      ## addproperty ("ticklength", hcb, "double", 0.01);
 
       ## Add a pointer from colorbar directly to axes
       addproperty ("__axes_handle__", hcb, "handle", hax);
@@ -394,6 +394,7 @@ endfunction
 
 ## Axes to which colorbar was attached is being deleted/reset. Delete colorbar.
 function cb_axes_deleted (~, ~, hax, hcb)
+
   if (isaxes (hcb))
     if (strcmp (get (hax, "beingdeleted"), "on"))
       ## Axes are being deleted.  Disable call to cb_restore_axes.
@@ -401,6 +402,7 @@ function cb_axes_deleted (~, ~, hax, hcb)
     endif
     delete (hcb);
   endif
+
 endfunction
 
 ## Error on attempt to set logscale on colorbar axes
@@ -440,7 +442,7 @@ function cb_restore_axes (hcb, ~, hax, orig_props)
     set (hax, "units", orig_props.units,
               "position", orig_props.position,
               "outerposition", orig_props.outerposition,
-              "activepositionproperty", orig_props.activepositionproperty);
+              "positionconstraint", orig_props.positionconstraint);
     set (hax, "units", units);
 
     ## Nullify colorbar link (can't delete properties yet)
@@ -471,7 +473,7 @@ function cb_clim (hax, ~, hcb, hi)
 endfunction
 
 ## Update colorbar when changes to axes or figure colormap have occurred.
-function cb_colormap (h, d, hax, hcb, hi, init_sz)
+function cb_colormap (h, ~, hax, hcb, hi, init_sz)
   persistent sz = init_sz;
 
   if (ishghandle (h))
@@ -536,7 +538,7 @@ function [axpos, cbpos, vertical, mirr] = calc_cbar_position (loc, props, cf)
       scale = [scale, 1];
     endif
     if (strcmp (get (cf, "__graphics_toolkit__"), "gnuplot")
-        && strcmp (props.activepositionproperty, "outerposition"))
+        && strcmp (props.positionconstraint, "outerposition"))
       props.outerposition = props.outerposition .* [1, 1, scale];
       off = 0.5 * (props.outerposition (3:4) - __actual_axis_position__ (props)(3:4));
     else
@@ -859,7 +861,7 @@ endfunction
 %!demo
 %! clf;
 %! colormap ("default");
-%! axes;
+%! axes ();
 %! colorbar ();
 %! hold on;
 %! contour (peaks ());
@@ -886,7 +888,7 @@ endfunction
 %! shading interp;
 %! axis ("tight", "square");
 %! colorbar ();
-#%! axes ("color","none","box","on","activepositionproperty","position");
+#%! axes ("color", "none", "box", "on", "positionconstraint", "innerposition");
 
 %!demo
 %! clf;
